@@ -1,6 +1,6 @@
 /*
  * Copyright (c) [2011-2014] Novell, Inc.
- * Copyright (c) 2016 SUSE LLC
+ * Copyright (c) [2016-2024] SUSE LLC
  *
  * All Rights Reserved.
  *
@@ -30,10 +30,16 @@
 #include "snapper/Snapper.h"
 
 #include "utils/Range.h"
+#include "utils/Limit.h"
 #include "utils/equal-date.h"
+#include "utils/HumanString.h"
+#include "proxy/locker.h"
 #include "cleanup.h"
 
 
+namespace snapper
+{
+
 using namespace std;
 
 
@@ -44,8 +50,19 @@ struct Parameters
 
     virtual bool is_degenerated() const { return true; }
 
-    time_t min_age;
-    double space_limit;
+    time_t min_age = 3600;
+    MaxUsedLimit space_limit = 0.5;
+    MinFreeLimit free_limit = 0.2;
+
+
+    void read(const ProxyConfig& config, const char* name, time_t& value)
+    {
+	const map<string, string>& raw = config.getAllValues();
+	map<string, string>::const_iterator pos = raw.find(name);
+	if (pos != raw.end())
+	    pos->second >> value;
+    }
+
 
     template<typename Type>
     void read(const ProxyConfig& config, const char* name, Type& value)
@@ -53,25 +70,37 @@ struct Parameters
 	const map<string, string>& raw = config.getAllValues();
 	map<string, string>::const_iterator pos = raw.find(name);
 	if (pos != raw.end())
-	    pos->second >> value;
+	{
+	    try
+	    {
+		value.parse(pos->second);
+	    }
+	    catch (const Exception& e)
+	    {
+		SN_CAUGHT(e);
+
+		cerr << "failed to parse \"" << pos->second << "\" for \"" << name << "\"" << endl;
+	    }
+	}
     }
 };
 
 
-Parameters::Parameters(const ProxySnapper* snapper)
-    : min_age(1800), space_limit(0.5)
-{
-    ProxyConfig config = snapper->getConfig();
-
-    read(config, "SPACE_LIMIT", space_limit);
-}
-
-
 ostream&
 operator<<(ostream& s, const Parameters& parameters)
 {
-    return s << "min-age:" << parameters.min_age << endl
-	     << "space-limit:" << parameters.space_limit;
+    return s << "min-age:" << parameters.min_age << '\n'
+	     << "space-limit:" << parameters.space_limit << '\n'
+	     << "free-limit:" << parameters.free_limit;
+}
+
+
+Parameters::Parameters(const ProxySnapper* snapper)
+{
+    ProxyConfig config = snapper->getConfig();
+
+    read(config, "SPACE_LIMIT", space_limit);
+    read(config, "FREE_LIMIT", free_limit);
 }
 
 
@@ -80,16 +109,17 @@ class Cleaner
 public:
 
     Cleaner(ProxySnapper* snapper, bool verbose, const Parameters& parameters)
-	: snapper(snapper), verbose(verbose), parameters(parameters) {}
+	: snapper(snapper), locker(snapper), verbose(verbose), parameters(parameters) {}
 
     virtual ~Cleaner() {}
 
-    void cleanup();
+    void cleanup(Plugins::Report& report);
+    void cleanup(std::function<bool()> condition, Plugins::Report& report);
 
 protected:
 
     virtual list<ProxySnapshots::iterator> calculate_candidates(ProxySnapshots& snapshots,
-								Range::Value value) = 0;
+								const Range::Value& value) = 0;
 
     struct younger_than
     {
@@ -102,6 +132,9 @@ protected:
 
     void filter(ProxySnapshots& snapshots, list<ProxySnapshots::iterator>& tmp) const;
 
+    // Removes snapshots that cannot be removed (e.g. btrfs active and default)
+    void filter_undeletables(ProxySnapshots& snapshots, list<ProxySnapshots::iterator>& tmp) const;
+
     // Removes snapshots younger than parameters.min_age from tmp
     void filter_min_age(ProxySnapshots& snapshots, list<ProxySnapshots::iterator>& tmp) const;
 
@@ -109,17 +142,28 @@ protected:
     // snapshot but which is not included in tmp.
     void filter_pre_post(ProxySnapshots& snapshots, list<ProxySnapshots::iterator>& tmp) const;
 
-    void remove(const list<ProxySnapshots::iterator>& tmp);
+    void remove(const list<ProxySnapshots::iterator>& tmp, Plugins::Report& report);
 
+    // Should the cleanup with quota space be run?
     bool is_quota_aware() const;
+
+    // Is the quota space condition satisfied?
     bool is_quota_satisfied() const;
 
-    void cleanup_quota_unaware(ProxySnapshots& snapshots);
-    void cleanup_quota_aware(ProxySnapshots& snapshots);
+    // Should the cleanup with free space be run?
+    bool is_free_aware() const;
+
+    // Is the free space condition satisfied?
+    bool is_free_satisfied() const;
+
+    void cleanup(ProxySnapshots& snapshots, Plugins::Report& report);
+    void cleanup(ProxySnapshots& snapshots, std::function<bool()> condition, Plugins::Report& report);
 
     ProxySnapper* snapper;
-    bool verbose;
 
+    Locker locker;
+
+    const bool verbose;
     const Parameters& parameters;
 
 };
@@ -128,11 +172,36 @@ protected:
 void
 Cleaner::filter(ProxySnapshots& snapshots, list<ProxySnapshots::iterator>& tmp) const
 {
+    filter_undeletables(snapshots, tmp);
     filter_min_age(snapshots, tmp);
     filter_pre_post(snapshots, tmp);
 }
 
 
+void
+Cleaner::filter_undeletables(ProxySnapshots& snapshots, list<ProxySnapshots::iterator>& tmp) const
+{
+    vector<ProxySnapshots::const_iterator> undeletables;
+
+    ProxySnapshots::const_iterator default_snapshot = snapshots.getDefault();
+    if (default_snapshot != snapshots.end())
+	undeletables.push_back(default_snapshot);
+
+    ProxySnapshots::const_iterator active_snapshot = snapshots.getActive();
+    if (active_snapshot != snapshots.end())
+	undeletables.push_back(active_snapshot);
+
+    for (ProxySnapshots::const_iterator undeletable : undeletables)
+    {
+	list<ProxySnapshots::iterator>::iterator keep = find_if(tmp.begin(), tmp.end(),
+	    [undeletable](ProxySnapshots::iterator it){ return undeletable->getNum() == it->getNum(); });
+
+	if (keep != tmp.end())
+	    tmp.erase(keep);
+    }
+}
+
+
 void
 Cleaner::filter_min_age(ProxySnapshots& snapshots, list<ProxySnapshots::iterator>& tmp) const
 {
@@ -176,11 +245,11 @@ Cleaner::filter_pre_post(ProxySnapshots& snapshots, list<ProxySnapshots::iterato
 
 
 void
-Cleaner::remove(const list<ProxySnapshots::iterator>& tmp)
+Cleaner::remove(const list<ProxySnapshots::iterator>& tmp, Plugins::Report& report)
 {
     for (list<ProxySnapshots::iterator>::const_iterator it = tmp.begin(); it != tmp.end(); ++it)
     {
-	snapper->deleteSnapshots({ *it }, verbose);
+	snapper->deleteSnapshots({ *it }, verbose, report);
     }
 }
 
@@ -195,18 +264,6 @@ Cleaner::is_quota_aware() const
     {
 	snapper->prepareQuota();
     }
-    catch (const DBus::ErrorException& e)
-    {
-	SN_CAUGHT(e);
-
-	if (strcmp(e.name(), "error.quota") == 0)
-	{
-	    cerr << "quota not working (" << e.message() << ")" << endl;
-	    return false;
-	}
-
-	SN_RETHROW(e);
-    }
     catch (const QuotaException& e)
     {
 	SN_CAUGHT(e);
@@ -215,7 +272,7 @@ Cleaner::is_quota_aware() const
 	return false;
     }
 
-    return true;
+    return parameters.space_limit.is_enabled();
 }
 
 
@@ -224,30 +281,88 @@ Cleaner::is_quota_satisfied() const
 {
     QuotaData quota_data = snapper->queryQuotaData();
 
-    return quota_data.used < parameters.space_limit * quota_data.size;
+    if (quota_data.size == 0)
+	return true;
+
+    bool satisfied = parameters.space_limit.is_satisfied(quota_data.size, quota_data.used);
+
+#ifdef VERBOSE_LOGGING
+    cout << byte_to_humanstring(quota_data.size, true, 2) << ", "
+	 << byte_to_humanstring(quota_data.used, true, 2) << ", "
+	 << satisfied << '\n';
+#endif
+
+    return satisfied;
+}
+
+
+bool
+Cleaner::is_free_aware() const
+{
+    if (parameters.is_degenerated())
+	return false;
+
+    try
+    {
+	snapper->queryFreeSpaceData();
+    }
+    catch (const FreeSpaceException& e)
+    {
+	SN_CAUGHT(e);
+
+	cerr << "free space not working (" << e.what() << ")" << endl;
+	return false;
+    }
+
+    return parameters.free_limit.is_enabled();
+}
+
+
+bool
+Cleaner::is_free_satisfied() const
+{
+    FreeSpaceData free_space_data = snapper->queryFreeSpaceData();
+
+    if (free_space_data.size == 0)
+	return true;
+
+    bool satisfied = parameters.free_limit.is_satisfied(free_space_data.size, free_space_data.free);
+
+#ifdef VERBOSE_LOGGING
+    cout << byte_to_humanstring(free_space_data.size, true, 2) << ", "
+	 << byte_to_humanstring(free_space_data.free, true, 2) << ", "
+	 << satisfied << '\n';
+#endif
+
+    return satisfied;
 }
 
 
 void
-Cleaner::cleanup_quota_unaware(ProxySnapshots& snapshots)
+Cleaner::cleanup(ProxySnapshots& snapshots, Plugins::Report& report)
 {
     list<ProxySnapshots::iterator> candidates = calculate_candidates(snapshots, Range::MAX);
 
     filter(snapshots, candidates);
 
-    remove(candidates);
+    remove(candidates, report);
 }
 
 
 void
-Cleaner::cleanup_quota_aware(ProxySnapshots& snapshots)
+Cleaner::cleanup(ProxySnapshots& snapshots, std::function<bool()> condition, Plugins::Report& report)
 {
-    while (!is_quota_satisfied())
+    while (!condition())
     {
 	list<ProxySnapshots::iterator> candidates = calculate_candidates(snapshots, Range::MIN);
 	if (candidates.empty())
 	{
-	    // not enough candidates to satisfy quota
+	    // not enough candidates to satisfy the condition
+
+#ifdef VERBOSE_LOGGING
+	    cout << "condition not satisfied" << '\n';
+#endif
+
 	    return;
 	}
 
@@ -264,30 +379,84 @@ Cleaner::cleanup_quota_aware(ProxySnapshots& snapshots)
 
 	    if (!tmp.empty())
 	    {
-		remove(tmp);
-		// after removing snapshots is_quota_satisfied must be reevaluated
+		remove(tmp, report);
+
+		// after removing snapshots the condition must be reevaluated
 		break;
 	    }
 
 	    if (next(e) == candidates.end())
 	    {
-		// not enough candidates to satisfy quota
+		// not enough candidates to satisfy the condition
+
+#ifdef VERBOSE_LOGGING
+		cout << "condition not satisfied" << '\n';
+#endif
+
 		return;
 	    }
 	}
     }
+
+#ifdef VERBOSE_LOGGING
+    cout << "condition satisfied" << '\n';
+#endif
+}
+
+
+void
+Cleaner::cleanup(Plugins::Report& report)
+{
+    ProxySnapshots& snapshots = snapper->getSnapshots();
+
+#ifdef VERBOSE_LOGGING
+    cout << "cleanup without condition" << '\n';
+#endif
+
+    cleanup(snapshots, report);
+
+    if (is_quota_aware())
+    {
+#ifdef VERBOSE_LOGGING
+	cout << "cleanup with quota condition" << '\n';
+#endif
+
+	cleanup(snapshots, std::bind(&Cleaner::is_quota_satisfied, this), report);
+    }
+    else
+    {
+#ifdef VERBOSE_LOGGING
+	cout << "no cleanup with quota condition" << '\n';
+#endif
+    }
+
+    if (is_free_aware())
+    {
+#ifdef VERBOSE_LOGGING
+	cout << "cleanup with free condition" << '\n';
+#endif
+
+	cleanup(snapshots, std::bind(&Cleaner::is_free_satisfied, this), report);
+    }
+    else
+    {
+#ifdef VERBOSE_LOGGING
+	cout << "no cleanup with free condition" << '\n';
+#endif
+    }
 }
 
 
 void
-Cleaner::cleanup()
+Cleaner::cleanup(std::function<bool()> condition, Plugins::Report& report)
 {
     ProxySnapshots& snapshots = snapper->getSnapshots();
 
-    cleanup_quota_unaware(snapshots);
+#ifdef VERBOSE_LOGGING
+    cout << "cleanup with user condition" << '\n';
+#endif
 
-    if (is_quota_aware())
-	cleanup_quota_aware(snapshots);
+    cleanup(snapshots, condition, report);
 }
 
 
@@ -295,13 +464,22 @@ struct NumberParameters : public Parameters
 {
     NumberParameters(const ProxySnapper* snapper);
 
-    bool is_degenerated() const;
+    bool is_degenerated() const override;
 
     Range limit;
     Range limit_important;
 };
 
 
+ostream&
+operator<<(ostream& s, const NumberParameters& parameters)
+{
+    return s << dynamic_cast<const Parameters&>(parameters) << '\n'
+	     << "limit:" << parameters.limit << '\n'
+	     << "limit-important:" << parameters.limit_important;
+}
+
+
 NumberParameters::NumberParameters(const ProxySnapper* snapper)
     : Parameters(snapper), limit(50), limit_important(10)
 {
@@ -311,15 +489,10 @@ NumberParameters::NumberParameters(const ProxySnapper* snapper)
 
     read(config, "NUMBER_LIMIT", limit);
     read(config, "NUMBER_LIMIT_IMPORTANT", limit_important);
-}
 
-
-ostream&
-operator<<(ostream& s, const NumberParameters& parameters)
-{
-    return s << dynamic_cast<const Parameters&>(parameters) << endl
-	     << "limit:" << parameters.limit << endl
-	     << "limit-important:" << parameters.limit_important;
+#ifdef VERBOSE_LOGGING
+    cout << *this << '\n';
+#endif
 }
 
 
@@ -349,7 +522,7 @@ private:
 
 
     list<ProxySnapshots::iterator>
-    calculate_candidates(ProxySnapshots& snapshots, Range::Value value) override
+    calculate_candidates(ProxySnapshots& snapshots, const Range::Value& value) override
     {
 	const NumberParameters& parameters = dynamic_cast<const NumberParameters&>(Cleaner::parameters);
 
@@ -394,11 +567,20 @@ private:
 
 
 void
-do_cleanup_number(ProxySnapper* snapper, bool verbose)
+do_cleanup_number(ProxySnapper* snapper, bool verbose, Plugins::Report& report)
 {
     NumberParameters parameters(snapper);
     NumberCleaner cleaner(snapper, verbose, parameters);
-    cleaner.cleanup();
+    cleaner.cleanup(report);
+}
+
+
+void
+do_cleanup_number(ProxySnapper* snapper, bool verbose, std::function<bool()> condition, Plugins::Report& report)
+{
+    NumberParameters parameters(snapper);
+    NumberCleaner cleaner(snapper, verbose, parameters);
+    cleaner.cleanup(condition, report);
 }
 
 
@@ -406,19 +588,33 @@ struct TimelineParameters : public Parameters
 {
     TimelineParameters(const ProxySnapper* snapper);
 
-    bool is_degenerated() const;
+    bool is_degenerated() const override;
 
     Range limit_hourly;
     Range limit_daily;
     Range limit_monthly;
     Range limit_weekly;
+    Range limit_quarterly;
     Range limit_yearly;
 };
 
 
+ostream&
+operator<<(ostream& s, const TimelineParameters& parameters)
+{
+    return s << dynamic_cast<const Parameters&>(parameters) << '\n'
+	     << "limit-hourly:" << parameters.limit_hourly << '\n'
+	     << "limit-daily:" << parameters.limit_daily << '\n'
+	     << "limit-weekly:" << parameters.limit_weekly << '\n'
+	     << "limit-monthly:" << parameters.limit_monthly << '\n'
+	     << "limit-quarterly:" << parameters.limit_quarterly << '\n'
+	     << "limit-yearly:" << parameters.limit_yearly;
+}
+
+
 TimelineParameters::TimelineParameters(const ProxySnapper* snapper)
     : Parameters(snapper), limit_hourly(10), limit_daily(10), limit_monthly(10),
-      limit_weekly(0), limit_yearly(10)
+      limit_weekly(0), limit_quarterly(0), limit_yearly(10)
 {
     ProxyConfig config = snapper->getConfig();
 
@@ -428,19 +624,12 @@ TimelineParameters::TimelineParameters(const ProxySnapper* snapper)
     read(config, "TIMELINE_LIMIT_DAILY", limit_daily);
     read(config, "TIMELINE_LIMIT_WEEKLY", limit_weekly);
     read(config, "TIMELINE_LIMIT_MONTHLY", limit_monthly);
+    read(config, "TIMELINE_LIMIT_QUARTERLY", limit_quarterly);
     read(config, "TIMELINE_LIMIT_YEARLY", limit_yearly);
-}
 
-
-ostream&
-operator<<(ostream& s, const TimelineParameters& parameters)
-{
-    return s << dynamic_cast<const Parameters&>(parameters) << endl
-	     << "limit-hourly:" << parameters.limit_hourly << endl
-	     << "limit-daily:" << parameters.limit_daily << endl
-	     << "limit-weekly:" << parameters.limit_weekly << endl
-	     << "limit-monthly:" << parameters.limit_monthly << endl
-	     << "limit-yearly:" << parameters.limit_yearly;
+#ifdef VERBOSE_LOGGING
+    cout << *this << '\n';
+#endif
 }
 
 
@@ -449,7 +638,7 @@ TimelineParameters::is_degenerated() const
 {
     return limit_hourly.is_degenerated() && limit_daily.is_degenerated() &&
 	limit_monthly.is_degenerated() && limit_weekly.is_degenerated() &&
-	limit_yearly.is_degenerated();
+	limit_quarterly.is_degenerated() && limit_yearly.is_degenerated();
 }
 
 
@@ -500,6 +689,14 @@ private:
 	return is_first(first, last, it1, equal_year);
     }
 
+    bool
+    is_first_quarterly(list<ProxySnapshots::iterator>::const_iterator first,
+		    list<ProxySnapshots::iterator>::const_iterator last,
+		    ProxySnapshots::const_iterator it1)
+    {
+	return is_first(first, last, it1, equal_quarter);
+    }
+
     bool
     is_first_monthly(list<ProxySnapshots::iterator>::const_iterator first,
 		     list<ProxySnapshots::iterator>::const_iterator last,
@@ -534,7 +731,7 @@ private:
 
 
     list<ProxySnapshots::iterator>
-    calculate_candidates(ProxySnapshots& snapshots, Range::Value value) override
+    calculate_candidates(ProxySnapshots& snapshots, const Range::Value& value) override
     {
 	const TimelineParameters& parameters = dynamic_cast<const TimelineParameters&>(Cleaner::parameters);
 
@@ -550,6 +747,7 @@ private:
 	size_t num_daily = 0;
 	size_t num_weekly = 0;
 	size_t num_monthly = 0;
+	size_t num_quarterly = 0;
 	size_t num_yearly = 0;
 
 	list<ProxySnapshots::iterator>::iterator it = ret.begin();
@@ -577,6 +775,11 @@ private:
 		++num_monthly;
 		keep = true;
 	    }
+	    if (num_quarterly < parameters.limit_quarterly.value(value) && is_first_quarterly(it, ret.end(), *it))
+	    {
+		++num_quarterly;
+		keep = true;
+	    }
 	    if (num_yearly < parameters.limit_yearly.value(value) && is_first_yearly(it, ret.end(), *it))
 	    {
 		++num_yearly;
@@ -597,11 +800,20 @@ private:
 
 
 void
-do_cleanup_timeline(ProxySnapper* snapper, bool verbose)
+do_cleanup_timeline(ProxySnapper* snapper, bool verbose, Plugins::Report& report)
 {
     TimelineParameters parameters(snapper);
     TimelineCleaner cleaner(snapper, verbose, parameters);
-    cleaner.cleanup();
+    cleaner.cleanup(report);
+}
+
+
+void
+do_cleanup_timeline(ProxySnapper* snapper, bool verbose, std::function<bool()> condition, Plugins::Report& report)
+{
+    TimelineParameters parameters(snapper);
+    TimelineCleaner cleaner(snapper, verbose, parameters);
+    cleaner.cleanup(condition, report);
 }
 
 
@@ -617,6 +829,10 @@ EmptyPrePostParameters::EmptyPrePostParameters(const ProxySnapper* snapper)
     ProxyConfig config = snapper->getConfig();
 
     read(config, "EMPTY_PRE_POST_MIN_AGE", min_age);
+
+#ifdef VERBOSE_LOGGING
+    cout << *this << '\n';
+#endif
 }
 
 
@@ -631,7 +847,7 @@ public:
 private:
 
     list<ProxySnapshots::iterator>
-    calculate_candidates(ProxySnapshots& snapshots, Range::Value value) override
+    calculate_candidates(ProxySnapshots& snapshots, const Range::Value& value) override
     {
 	list<ProxySnapshots::iterator> ret;
 
@@ -658,9 +874,21 @@ private:
 
 
 void
-do_cleanup_empty_pre_post(ProxySnapper* snapper, bool verbose)
+do_cleanup_empty_pre_post(ProxySnapper* snapper, bool verbose, Plugins::Report& report)
 {
     EmptyPrePostParameters parameters(snapper);
     EmptyPrePostCleaner cleaner(snapper, verbose, parameters);
-    cleaner.cleanup();
+    cleaner.cleanup(report);
+}
+
+
+void
+do_cleanup_empty_pre_post(ProxySnapper* snapper, bool verbose, std::function<bool()> condition,
+			  Plugins::Report& report)
+{
+    EmptyPrePostParameters parameters(snapper);
+    EmptyPrePostCleaner cleaner(snapper, verbose, parameters);
+    cleaner.cleanup(condition, report);
+}
+
 }
