@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016 SUSE LLC
+ * Copyright (c) [2016-2021] SUSE LLC
  *
  * All Rights Reserved.
  *
@@ -20,32 +20,48 @@
  */
 
 
+#ifndef SNAPPER_RANGE_H
+#define SNAPPER_RANGE_H
+
+
+#include <string>
 #include <ostream>
 
-using std::istream;
-using std::ostream;
 
-
-/*
- * Simple class to hold a range of two size_ts as min and max.
- */
-class Range
+namespace snapper
 {
-public:
+    using namespace std;
 
-    enum Value { MIN, MAX };
 
-    Range(size_t value) : min(value), max(value) {}
+    /*
+     * Simple class to hold a range of two size_ts as min and max.
+     */
+    class Range
+    {
+    public:
 
-    size_t value(Value value) const { return value == MIN ? min : max; }
+	enum Value { MIN, MAX };
 
-    bool is_degenerated() const { return min == max; }
+	Range(size_t value) : min(value), max(value) {}
 
-    friend istream& operator>>(istream& s, Range& range);
-    friend ostream& operator<<(ostream& s, const Range& range);
+	/**
+	 * Uses the classic locale since the class is intended for config data.
+	 */
+	void parse(const string& s);
 
-private:
+	size_t value(Value value) const { return value == MIN ? min : max; }
 
-    size_t min;
-    size_t max;
-};
+	bool is_degenerated() const { return min == max; }
+
+	friend ostream& operator<<(ostream& s, const Range& range);
+
+    private:
+
+	size_t min;
+	size_t max;
+
+    };
+
+}
+
+#endif
