@@ -1,6 +1,6 @@
 /*
  * Copyright (c) [2011-2015] Novell, Inc.
- * Copyright (c) [2016-2024] SUSE LLC
+ * Copyright (c) [2016-2025] SUSE LLC
  *
  * All Rights Reserved.
  *
@@ -41,13 +41,12 @@
 #include <btrfs/send.h>
 #include <btrfs/send-stream.h>
 #include <btrfs/send-utils.h>
-#include <boost/version.hpp>
 #include <boost/thread.hpp>
 #endif
 #include <regex>
 #include <boost/algorithm/string.hpp>
 
-#include "snapper/Log.h"
+#include "snapper/LoggerImpl.h"
 #include "snapper/Btrfs.h"
 #include "snapper/BtrfsUtils.h"
 #include "snapper/File.h"
@@ -70,11 +69,11 @@ namespace snapper
     using namespace std;
 
 
-    Filesystem*
+    std::unique_ptr<Filesystem>
     Btrfs::create(const string& fstype, const string& subvolume, const string& root_prefix)
     {
 	if (fstype == "btrfs")
-	    return new Btrfs(subvolume, root_prefix);
+	    return std::make_unique<Btrfs>(subvolume, root_prefix);
 
 	return nullptr;
     }
@@ -461,10 +460,14 @@ namespace snapper
 
 
     void
-    Btrfs::setSnapshotReadOnly(unsigned int num, bool read_only) const
+    Btrfs::setSnapshotReadOnly(unsigned int num, bool read_only, Plugins::Report& report) const
     {
+	Plugins::set_read_only(Plugins::Stage::PRE_ACTION, subvolume, this, num, report);
+
 	SDir snapshot_dir = openSnapshotDir(num);
 	set_subvolume_read_only(snapshot_dir.fd(), read_only);
+
+	Plugins::set_read_only(Plugins::Stage::POST_ACTION, subvolume, this, num, report);
     }
 
 
@@ -768,12 +771,15 @@ namespace snapper
     bool
     StreamProcessor::get_root_id(const string& path, u64* root_id)
     {
-	struct subvol_info* si;
-	si = subvol_uuid_search(&sus, 0, NULL, 0, path.c_str(), subvol_search_by_path);
+	struct subvol_info* si = subvol_uuid_search(&sus, 0, NULL, 0, path.c_str(), subvol_search_by_path);
 	if (!si)
 	    return false;
 
 	*root_id = si->root_id;
+
+	free(si->path);
+	free(si);
+
 	return true;
     }
 
@@ -1465,14 +1471,6 @@ namespace snapper
     }
 
 
-    void
-    Btrfs::setDefault(unsigned int num) const
-    {
-	Plugins::Report report;
-	setDefault(num, report);
-    }
-
-
     void
     Btrfs::setDefault(unsigned int num, Plugins::Report& report) const
     {
@@ -1552,13 +1550,6 @@ namespace snapper
     }
 
 
-    void
-    Btrfs::setDefault(unsigned int num) const
-    {
-	Filesystem::setDefault(num);
-    }
-
-
     void
     Btrfs::setDefault(unsigned int num, Plugins::Report& report) const
     {
