@@ -1,6 +1,6 @@
 /*
  * Copyright (c) [2011-2015] Novell, Inc.
- * Copyright (c) [2016-2017] SUSE LLC
+ * Copyright (c) [2016-2024] SUSE LLC
  *
  * All Rights Reserved.
  *
@@ -23,22 +23,25 @@
 
 #include "config.h"
 
-#include <string.h>
+#include <cstring>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/mount.h>
-#include <errno.h>
+#include <cerrno>
 #include <unistd.h>
 #include <mntent.h>
 #include <fcntl.h>
 #include <asm/types.h>
 #include <boost/algorithm/string.hpp>
 
-#include "snapper/Log.h"
+#include "snapper/LoggerImpl.h"
 #include "snapper/Filesystem.h"
 #ifdef ENABLE_BTRFS
 #include "snapper/Btrfs.h"
 #endif
+#ifdef ENABLE_BCACHEFS
+#include "snapper/Bcachefs.h"
+#endif
 #ifdef ENABLE_EXT4
 #include "snapper/Ext4.h"
 #endif
@@ -57,7 +60,7 @@ namespace snapper
     vector<string>
     Filesystem::filter_mount_options(const vector<string>& options)
     {
-	static const char* ign_opt[] = {
+	static const char* ign_opts[] = {
 	    "ro", "rw",
 	    "exec", "noexec", "suid", "nosuid", "dev", "nodev",
 	    "atime", "noatime", "diratime", "nodiratime",
@@ -66,8 +69,8 @@ namespace snapper
 
 	vector<string> ret = options;
 
-	for (size_t i = 0; i < lengthof(ign_opt); ++i)
-	    ret.erase(remove(ret.begin(), ret.end(), ign_opt[i]), ret.end());
+	for (const char* ign_opt : ign_opts)
+	    ret.erase(remove(ret.begin(), ret.end(), ign_opt), ret.end());
 
 	return ret;
     }
@@ -91,28 +94,31 @@ namespace snapper
     }
 
 
-    Filesystem*
+    std::unique_ptr<Filesystem>
     Filesystem::create(const string& fstype, const string& subvolume, const string& root_prefix)
     {
-	typedef Filesystem* (*func_t)(const string& fstype, const string& subvolume,
-				      const string& root_prefix);
+	typedef std::unique_ptr<Filesystem> (*func_t)(const string& fstype, const string& subvolume,
+						      const string& root_prefix);
 
 	static const func_t funcs[] = {
 #ifdef ENABLE_BTRFS
 		&Btrfs::create,
 #endif
+#ifdef ENABLE_BCACHEFS
+		&Bcachefs::create,
+#endif
 #ifdef ENABLE_EXT4
 		&Ext4::create,
 #endif
 #ifdef ENABLE_LVM
 		&Lvm::create,
 #endif
-		NULL
+		nullptr
 	};
 
-	for (const func_t* func = funcs; *func != NULL; ++func)
+	for (const func_t* func = funcs; *func != nullptr; ++func)
 	{
-	    Filesystem* fs = (*func)(fstype, subvolume, root_prefix);
+	    std::unique_ptr<Filesystem> fs = (*func)(fstype, subvolume, root_prefix);
 	    if (fs)
 		return fs;
 	}
@@ -123,13 +129,13 @@ namespace snapper
     }
 
 
-    Filesystem*
+    std::unique_ptr<Filesystem>
     Filesystem::create(const ConfigInfo& config_info, const string& root_prefix)
     {
 	string fstype = "btrfs";
-	config_info.getValue(KEY_FSTYPE, fstype);
+	config_info.get_value(KEY_FSTYPE, fstype);
 
-	Filesystem* fs = create(fstype, config_info.getSubvolume(), root_prefix);
+	std::unique_ptr<Filesystem> fs = create(fstype, config_info.get_subvolume(), root_prefix);
 
 	fs->evalConfigInfo(config_info);
 
@@ -166,35 +172,44 @@ namespace snapper
     void
     Filesystem::createSnapshotOfDefault(unsigned int num, bool read_only, bool quota) const
     {
-	throw std::logic_error("not implemented");
+	SN_THROW(UnsupportedException());
+	__builtin_unreachable();
     }
 
 
     bool
     Filesystem::isDefault(unsigned int num) const
     {
-	throw std::logic_error("not implemented");
+	return false;
     }
 
 
     std::pair<bool, unsigned int>
     Filesystem::getDefault() const
     {
-	throw std::logic_error("not implemented");
+	return std::make_pair(false, 0);
     }
 
 
     void
-    Filesystem::setDefault(unsigned int num) const
+    Filesystem::setDefault(unsigned int num, Plugins::Report& report) const
     {
-	throw std::logic_error("not implemented");
+	SN_THROW(UnsupportedException());
+	__builtin_unreachable();
+    }
+
+
+    std::pair<bool, unsigned int>
+    Filesystem::getActive() const
+    {
+	return std::make_pair(false, 0);
     }
 
 
     bool
     Filesystem::isActive(unsigned int num) const
     {
-	throw std::logic_error("not implemented");
+	return false;
     }
 
 
