@@ -1,6 +1,6 @@
 /*
  * Copyright (c) [2011-2015] Novell, Inc.
- * Copyright (c) [2016-2024] SUSE LLC
+ * Copyright (c) [2016-2025] SUSE LLC
  *
  * All Rights Reserved.
  *
@@ -63,46 +63,17 @@ namespace snapper
     using namespace std;
 
 
-    ConfigInfo::ConfigInfo(const string& config_name, const string& root_prefix)
-	: SysconfigFile(prepend_root_prefix(root_prefix, CONFIGS_DIR "/" + config_name)),
-	  config_name(config_name), subvolume("/")
-    {
-	if (!get_value(KEY_SUBVOLUME, subvolume))
-	    SN_THROW(InvalidConfigException());
-    }
-
-
-    void
-    ConfigInfo::check_key(const string& key) const
-    {
-	if (key == KEY_SUBVOLUME || key == KEY_FSTYPE)
-	    SN_THROW(InvalidConfigdataException());
-
-	try
-	{
-	    SysconfigFile::check_key(key);
-	}
-	catch (const InvalidKeyException& e)
-	{
-	    SN_CAUGHT(e);
-
-	    SN_THROW(InvalidConfigdataException());
-	}
-    }
-
-
     Snapper::Snapper(const string& config_name, const string& root_prefix, bool disable_filters)
-	: snapshots(this)
+	: config_name(config_name), root_prefix(root_prefix), snapshots(this)
     {
-	y2mil("Snapper constructor");
+	y2mil("Snapper constructor '" << config_name << "'");
 	y2mil("snapper version " VERSION);
 	y2mil("libsnapper version " LIBSNAPPER_VERSION_STRING);
-	y2mil("config_name:" << config_name << " root_prefix:" << root_prefix <<
-	      " disable_filters:" << disable_filters);
+	y2mil("root-prefix:" << root_prefix << " disable-filters:" << disable_filters);
 
 	try
 	{
-	    config_info = new ConfigInfo(config_name, root_prefix);
+	    config_info = make_unique<ConfigInfo>(config_name, root_prefix);
 	}
 	catch (const Exception& e)
 	{
@@ -139,25 +110,19 @@ namespace snapper
 
     Snapper::~Snapper()
     {
-	y2mil("Snapper destructor");
+	y2mil("Snapper destructor '" << config_name << "'");
 
-	for (Snapshots::iterator it = snapshots.begin(); it != snapshots.end(); ++it)
+	for (const Snapshot& snapshot : snapshots)
 	{
 	    try
 	    {
-		it->handleUmountFilesystemSnapshot();
+		snapshot.handleUmountFilesystemSnapshot();
 	    }
 	    catch (const UmountSnapshotFailedException& e)
 	    {
 		SN_CAUGHT(e);
 	    }
 	}
-
-	delete filesystem;
-	filesystem = nullptr;
-
-	delete config_info;
-	config_info = nullptr;
     }
 
 
@@ -234,62 +199,6 @@ namespace snapper
     }
 
 
-    Snapshots::iterator
-    Snapper::createSingleSnapshot(const SCD& scd)
-    {
-	Plugins::Report report;
-	return createSingleSnapshot(scd, report);
-    }
-
-
-    Snapshots::iterator
-    Snapper::createSingleSnapshot(Snapshots::const_iterator parent, const SCD& scd)
-    {
-	Plugins::Report report;
-	return createSingleSnapshot(parent, scd, report);
-    }
-
-
-    Snapshots::iterator
-    Snapper::createSingleSnapshotOfDefault(const SCD& scd)
-    {
-	Plugins::Report report;
-	return createSingleSnapshotOfDefault(scd, report);
-    }
-
-
-    Snapshots::iterator
-    Snapper::createPreSnapshot(const SCD& scd)
-    {
-	Plugins::Report report;
-	return createPreSnapshot(scd, report);
-    }
-
-
-    Snapshots::iterator
-    Snapper::createPostSnapshot(Snapshots::const_iterator pre, const SCD& scd)
-    {
-	Plugins::Report report;
-	return createPostSnapshot(pre, scd, report);
-    }
-
-
-    void
-    Snapper::modifySnapshot(Snapshots::iterator snapshot, const SMD& smd)
-    {
-	Plugins::Report report;
-	modifySnapshot(snapshot, smd, report);
-    }
-
-
-    void
-    Snapper::deleteSnapshot(Snapshots::iterator snapshot)
-    {
-	Plugins::Report report;
-	deleteSnapshot(snapshot, report);
-    }
-
-
     Snapshots::iterator
     Snapper::createSingleSnapshot(const SCD& scd, Plugins::Report& report)
     {
@@ -388,16 +297,6 @@ namespace snapper
     }
 
 
-    void
-    Snapper::createConfig(const string& config_name, const string& root_prefix,
-			  const string& subvolume, const string& fstype,
-			  const string& template_name)
-    {
-	Plugins::Report report;
-	createConfig(config_name, root_prefix, subvolume, fstype, template_name, report);
-    }
-
-
     void
     Snapper::createConfig(const string& config_name, const string& root_prefix,
 			  const string& subvolume, const string& fstype,
@@ -418,10 +317,10 @@ namespace snapper
 	    SN_THROW(CreateConfigFailedException("illegal subvolume"));
 	}
 
-	list<ConfigInfo> configs = getConfigs(root_prefix);
-	for (list<ConfigInfo>::const_iterator it = configs.begin(); it != configs.end(); ++it)
+	list<ConfigInfo> config_infos = getConfigs(root_prefix);
+	for (const ConfigInfo& config_info : config_infos)
 	{
-	    if (it->get_subvolume() == subvolume)
+	    if (config_info.get_subvolume() == subvolume)
 	    {
 		SN_THROW(CreateConfigFailedException("subvolume already covered"));
 	    }
@@ -441,7 +340,7 @@ namespace snapper
 	unique_ptr<Filesystem> filesystem;
 	try
 	{
-	    filesystem.reset(Filesystem::create(fstype, subvolume, ""));
+	    filesystem = Filesystem::create(fstype, subvolume, "");
 	}
 	catch (const InvalidConfigException& e)
 	{
@@ -533,21 +432,13 @@ namespace snapper
     }
 
 
-    void
-    Snapper::deleteConfig(const string& config_name, const string& root_prefix)
-    {
-	Plugins::Report report;
-	deleteConfig(config_name, root_prefix, report);
-    }
-
-
     void
     Snapper::deleteConfig(const string& config_name, const string& root_prefix, Plugins::Report& report)
     {
 	y2mil("Snapper delete-config");
 	y2mil("libsnapper version " VERSION);
 
-	unique_ptr<Snapper> snapper(new Snapper(config_name, root_prefix));
+	unique_ptr<Snapper> snapper = make_unique<Snapper>(config_name, root_prefix);
 
 	Plugins::delete_config(Plugins::Stage::PRE_ACTION, snapper->subvolumeDir(), snapper->getFilesystem(),
 			       report);
